/*
* Copyright 2019,2023 NXP
* All rights reserved.
*
* SPDX-License-Identifier: BSD-3-Clause
*/


/****************************************************************************/
/***        Include files                                                 ***/
/****************************************************************************/
#include <EmbeddedTypes.h>
#include "dbg.h"
#include "app.h"
#include "app_leds.h"
#include "zb_platform.h"

/****************************************************************************/
/***        Macro Definitions                                             ***/
/****************************************************************************/

/****************************************************************************/
/***        Type Definitions                                              ***/
/****************************************************************************/
/****************************************************************************/
/***        Local Function Prototypes                                     ***/
/****************************************************************************/


/****************************************************************************/
/***        Exported Variables                                            ***/
/****************************************************************************/

/****************************************************************************/
/***        Local Variables                                               ***/
/****************************************************************************/

/****************************************************************************/
/***        Exported Functions                                            ***/
/****************************************************************************/
/****************************************************************************
 *
 * NAME: APP_vLedInitialise
 *
 * DESCRIPTION:
 * initialise the application leds
 *
 * PARAMETER: void
 *
 * RETURNS: void
 *
 ****************************************************************************/
void APP_vLedInitialise(void)
{
/* Define the init structure for the output LED pin*/
    gpio_pin_config_t led_config = {
        kGPIO_DigitalOutput, OFF,
    };

    GPIO_PinInit(APP_LED1_GPIO, APP_LED1_PIN, &led_config);
    GPIO_PinInit(APP_LED2_GPIO, APP_LED2_PIN, &led_config);
    GPIO_PinInit(APP_LED3_GPIO, APP_LED3_PIN, &led_config);
}

/****************************************************************************
 *
 * NAME: APP_vSetLed
 *
 * DESCRIPTION:
 * set the state ofthe given application led
 *
 * PARAMETER: void
 *
 * RETURNS: void
 *
 ****************************************************************************/
void APP_vSetLed(uint8_t u8Led, bool_t bState)
{
    switch (u8Led)
    {
    case LED1:
        GPIO_PinWrite(APP_LED1_GPIO, APP_LED1_PIN, bState);
        break;
    case LED2:
        GPIO_PinWrite(APP_LED2_GPIO, APP_LED2_PIN, bState);
        break;
    case LED3:
        GPIO_PinWrite(APP_LED3_GPIO, APP_LED3_PIN, bState);
        break;
    default:
        break;
    }
}

/****************************************************************************
 *
 * NAME: APP_u8GetLedStates
 *
 * DESCRIPTION:
 * Get leds state as bit mask
 *
 * PARAMETER: void
 *
 * RETURNS: uint8_t bitmask
 *
 ****************************************************************************/
uint8_t APP_u8GetLedStates(void)
{
	//return zbPlatLedGetStates();
	return 0;
}

/****************************************************************************/
/***        Local Functions                                               ***/
/****************************************************************************/


/****************************************************************************/
/***        END OF FILE                                                   ***/
/****************************************************************************/
