/*
 * Copyright 2021 NXP
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */

/*${header:start}*/
#include "board.h"
#include "board_dcdc.h"
#include "clock_config.h"
#include "fwk_platform.h"
#include "fwk_debug.h"
#include "pin_mux.h"
#include "app_leds.h"

#include "jendefs.h"
#include "fsl_common.h"
#include "fsl_port.h"
#include "fsl_gpio.h"
#include "app_buttons.h"
#include "fsl_wuu.h"
#include "fsl_spc.h"
#include "app.h"

#include "app_main.h"

#if (defined GpEhSwitch) && (defined K32W148EVK)
#include "app_nvm.h"
#endif

#if defined(gBoardUseFro32k_d) && (gBoardUseFro32k_d > 0)
#include "fwk_sfc.h"
#endif
/*${header:end}*/

#if defined(gBoardUseFro32k_d) && (gBoardUseFro32k_d > 0)
static sfc_config_t sfcConfig;
#endif

void BOARD_InitPinLED1_OM15082(void)
{
    /* Clock Configuration: Peripheral clocks are enabled; module does not stall low power mode entry */
    CLOCK_EnableClock(kCLOCK_PortA);

    const port_pin_config_t OM15082_LED1 = {/* Internal pull-up resistor is enabled */
                                   (uint16_t)kPORT_PullDisable,
                                   /* Low internal pull resistor value is selected. */
                                   (uint16_t)kPORT_LowPullResistor,
                                   /* Slow slew rate is configured */
                                   (uint16_t)kPORT_SlowSlewRate,
                                   /* Passive input filter is disabled */
                                   (uint16_t)kPORT_PassiveFilterDisable,
                                   /* Open drain output is enabled */
                                   (uint16_t)kPORT_OpenDrainDisable,
                                   /* Low drive strength is configured */
                                   (uint16_t)kPORT_LowDriveStrength,
                                   /* Normal drive strength is configured */
                                   (uint16_t)kPORT_NormalDriveStrength,
                                   /* Pin is configured as PTC6 */
                                   (uint16_t)kPORT_MuxAsGpio,
                                   /* Pin Control Register fields [15:0] are not locked */
                                   (uint16_t)kPORT_UnlockRegister};
    PORT_SetPinConfig(APP_LED1_PORT, APP_LED1_PIN, &OM15082_LED1);
}
void BOARD_InitPinLED2_OM15082(void)
{
    /* Clock Configuration: Peripheral clocks are enabled; module does not stall low power mode entry */
    CLOCK_EnableClock(kCLOCK_PortC);

    const port_pin_config_t OM15082_LED2 = {/* Internal pull-up resistor is enabled */
                                   (uint16_t)kPORT_PullDisable,
                                   /* Low internal pull resistor value is selected. */
                                   (uint16_t)kPORT_LowPullResistor,
                                   /* Slow slew rate is configured */
                                   (uint16_t)kPORT_SlowSlewRate,
                                   /* Passive input filter is disabled */
                                   (uint16_t)kPORT_PassiveFilterDisable,
                                   /* Open drain output is enabled */
                                   (uint16_t)kPORT_OpenDrainDisable,
                                   /* Low drive strength is configured */
                                   (uint16_t)kPORT_LowDriveStrength,
                                   /* Normal drive strength is configured */
                                   (uint16_t)kPORT_NormalDriveStrength,
                                   /* Pin is configured as PTC6 */
                                   (uint16_t)kPORT_MuxAsGpio,
                                   /* Pin Control Register fields [15:0] are not locked */
                                   (uint16_t)kPORT_UnlockRegister};
    PORT_SetPinConfig(APP_LED2_PORT, APP_LED2_PIN, &OM15082_LED2);
}
void BOARD_InitPinLED3_OM15082(void)
{
    /* Clock Configuration: Peripheral clocks are enabled; module does not stall low power mode entry */
    CLOCK_EnableClock(kCLOCK_PortA);

    const port_pin_config_t OM15082_LED3 = {/* Internal pull-up resistor is enabled */
                                   (uint16_t)kPORT_PullDisable,
                                   /* Low internal pull resistor value is selected. */
                                   (uint16_t)kPORT_LowPullResistor,
                                   /* Slow slew rate is configured */
                                   (uint16_t)kPORT_SlowSlewRate,
                                   /* Passive input filter is disabled */
                                   (uint16_t)kPORT_PassiveFilterDisable,
                                   /* Open drain output is enabled */
                                   (uint16_t)kPORT_OpenDrainDisable,
                                   /* Low drive strength is configured */
                                   (uint16_t)kPORT_LowDriveStrength,
                                   /* Normal drive strength is configured */
                                   (uint16_t)kPORT_NormalDriveStrength,
                                   /* Pin is configured as PTC6 */
                                   (uint16_t)kPORT_MuxAsGpio,
                                   /* Pin Control Register fields [15:0] are not locked */
                                   (uint16_t)kPORT_UnlockRegister};
    PORT_SetPinConfig(APP_LED3_PORT, APP_LED3_PIN, &OM15082_LED3);
}

void BOARD_InitPinButton1_OM15082(void)
{
    /* Clock Configuration: Peripheral clocks are enabled; module does not stall low power mode entry */
    CLOCK_EnableClock(kCLOCK_PortC);

    const port_pin_config_t SW1 = {/* Internal pull-up resistor is enabled */
                                   (uint16_t)kPORT_PullUp,
                                   /* Low internal pull resistor value is selected. */
                                   (uint16_t)kPORT_LowPullResistor,
                                   /* Slow slew rate is configured */
                                   (uint16_t)kPORT_SlowSlewRate,
                                   /* Passive input filter is disabled */
                                   (uint16_t)kPORT_PassiveFilterDisable,
                                   /* Open drain output is enabled */
                                   (uint16_t)kPORT_OpenDrainEnable,
                                   /* Low drive strength is configured */
                                   (uint16_t)kPORT_LowDriveStrength,
                                   /* Normal drive strength is configured */
                                   (uint16_t)kPORT_NormalDriveStrength,
                                   /* Pin is configured as PTC6 */
                                   (uint16_t)kPORT_MuxAsGpio,
                                   /* Pin Control Register fields [15:0] are not locked */
                                   (uint16_t)kPORT_UnlockRegister};
    PORT_SetPinConfig(APP_BUTTONS_BUTTON_1_PORT, APP_BUTTONS_BUTTON_1_PIN, &SW1);
}

void BOARD_InitPinButton2_OM15082(void)
{

    const port_pin_config_t OM15082_SW2 = {/* Internal pull-up resistor is enabled */
                                   (uint16_t)kPORT_PullUp,
                                   /* High internal pull resistor value is selected. */
                                   (uint16_t)kPORT_HighPullResistor,
                                   /* Fast slew rate is configured */
                                   (uint16_t)kPORT_FastSlewRate,
                                   /* Passive input filter is disabled */
                                   (uint16_t)kPORT_PassiveFilterDisable,
                                   /* Open drain output is enabled */
                                   (uint16_t)kPORT_OpenDrainEnable,
                                   /* Low drive strength is configured */
                                   (uint16_t)kPORT_LowDriveStrength,
                                   /* Normal drive strength is configured */
                                   (uint16_t)kPORT_NormalDriveStrength,
                                   /* Pin is configured as PTD1 */
                                   (uint16_t)kPORT_MuxAsGpio,
                                   /* Pin Control Register fields [15:0] are not locked */
                                   (uint16_t)kPORT_UnlockRegister};
    PORT_SetPinConfig(APP_BUTTONS_BUTTON_2_PORT, APP_BUTTONS_BUTTON_2_PIN, &OM15082_SW2);
}

void BOARD_InitPinButton3_OM15082(void)
{

    const port_pin_config_t OM15082_SW3 = {/* Internal pull-up resistor is enabled */
                                   (uint16_t)kPORT_PullUp,
                                   /* High internal pull resistor value is selected. */
                                   (uint16_t)kPORT_HighPullResistor,
                                   /* Fast slew rate is configured */
                                   (uint16_t)kPORT_FastSlewRate,
                                   /* Passive input filter is disabled */
                                   (uint16_t)kPORT_PassiveFilterDisable,
                                   /* Open drain output is enabled */
                                   (uint16_t)kPORT_OpenDrainEnable,
                                   /* Low drive strength is configured */
                                   (uint16_t)kPORT_LowDriveStrength,
                                   /* Normal drive strength is configured */
                                   (uint16_t)kPORT_NormalDriveStrength,
                                   /* Pin is configured as PTD1 */
                                   (uint16_t)kPORT_MuxAsGpio,
                                   /* Pin Control Register fields [15:0] are not locked */
                                   (uint16_t)kPORT_UnlockRegister};
    PORT_SetPinConfig(APP_BUTTONS_BUTTON_3_PORT, APP_BUTTONS_BUTTON_3_PIN, &OM15082_SW3);
}

void BOARD_InitPinButton4_OM15082(void)
{

    const port_pin_config_t OM15082_SW4 = {/* Internal pull-up resistor is enabled */
                                   (uint16_t)kPORT_PullUp,
                                   /* High internal pull resistor value is selected. */
                                   (uint16_t)kPORT_HighPullResistor,
                                   /* Fast slew rate is configured */
                                   (uint16_t)kPORT_FastSlewRate,
                                   /* Passive input filter is disabled */
                                   (uint16_t)kPORT_PassiveFilterDisable,
                                   /* Open drain output is enabled */
                                   (uint16_t)kPORT_OpenDrainEnable,
                                   /* Low drive strength is configured */
                                   (uint16_t)kPORT_LowDriveStrength,
                                   /* Normal drive strength is configured */
                                   (uint16_t)kPORT_NormalDriveStrength,
                                   /* Pin is configured as PTD1 */
                                   (uint16_t)kPORT_MuxAsGpio,
                                   /* Pin Control Register fields [15:0] are not locked */
                                   (uint16_t)kPORT_UnlockRegister};
    PORT_SetPinConfig(APP_BUTTONS_BUTTON_4_PORT, APP_BUTTONS_BUTTON_4_PIN, &OM15082_SW4);
}

void BOARD_UnInitPinLED1_OM15082(void)
{
    /* Clock Configuration: Peripheral clocks are enabled; module does not stall low power mode entry */
    CLOCK_EnableClock(kCLOCK_PortA);

    PORT_SetPinMux(APP_LED1_PORT, APP_LED1_PIN, kPORT_PinDisabledOrAnalog);
}
void BOARD_UnInitPinLED2_OM15082(void)
{
    /* Clock Configuration: Peripheral clocks are enabled; module does not stall low power mode entry */
    CLOCK_EnableClock(kCLOCK_PortC);

    PORT_SetPinMux(APP_LED2_PORT, APP_LED2_PIN, kPORT_PinDisabledOrAnalog);
}
void BOARD_UnInitPinLED3_OM15082(void)
{
    /* Clock Configuration: Peripheral clocks are enabled; module does not stall low power mode entry */
    CLOCK_EnableClock(kCLOCK_PortA);

    PORT_SetPinMux(APP_LED3_PORT, APP_LED3_PIN, kPORT_PinDisabledOrAnalog);
}

void BOARD_UnInitPinButton1_OM15082(void)
{
    /* Clock Configuration: Peripheral clocks are enabled; module does not stall low power mode entry */
    CLOCK_EnableClock(kCLOCK_PortC);

    PORT_SetPinMux(APP_BUTTONS_BUTTON_1_PORT, APP_BUTTONS_BUTTON_1_PIN, kPORT_PinDisabledOrAnalog);
}

void BOARD_UnInitPinButton2_OM15082(void)
{
    PORT_SetPinMux(APP_BUTTONS_BUTTON_2_PORT, APP_BUTTONS_BUTTON_2_PIN, kPORT_PinDisabledOrAnalog);
}

void BOARD_UnInitPinButton3_OM15082(void)
{
    PORT_SetPinMux(APP_BUTTONS_BUTTON_3_PORT, APP_BUTTONS_BUTTON_3_PIN, kPORT_PinDisabledOrAnalog);
}

void BOARD_UnInitPinButton4_OM15082(void)
{
    PORT_SetPinMux(APP_BUTTONS_BUTTON_4_PORT, APP_BUTTONS_BUTTON_4_PIN, kPORT_PinDisabledOrAnalog);
}

void BOARD_SetPinsForPowerMode(void)
{
    BOARD_UnInitPinLPUART1_RX();
    BOARD_UnInitPinLPUART1_TX();

#ifdef IO_RETENTION_TEST
    /*set LED state to 1, this should make the three leds ON*/
    APP_vSetLED(LED1, 1);
    APP_vSetLED(LED2, 1);
    APP_vSetLED(LED3, 1);
#else
    BOARD_UnInitPinLED1_OM15082();
    BOARD_UnInitPinLED2_OM15082();
    BOARD_UnInitPinLED3_OM15082();
#endif

#ifdef DISABLE_DIO_WAKEUP
    /*Use PTC6 for wakeup, so both IRQ and Pin state must be kept during sleep*/
    /*For test purpose, uncomment below line if you use timer wakeup only*/
    BOARD_UnInitPinButton1(); 
#endif

#if (defined LOW_CURRENT_TEST_MODE) || (defined DISABLE_DIO_WAKEUP)
    BOARD_UnInitPinButton1_OM15082();
    BOARD_UnInitPinButton2_OM15082();
    BOARD_UnInitPinButton3_OM15082();
    BOARD_UnInitPinButton4_OM15082();
#endif

    /*For K32W148EVK board low power, see [KFOURWONE-3537]*/
    BOARD_InitPins();
}

/*${function:start}*/
void BOARD_InitHardware(void)
{
    BOARD_DBGCONFIGINIT(TRUE); // internal debug

#if defined(gAppHighSystemClockFrequency_d) && (gAppHighSystemClockFrequency_d > 0)
    /* Set Core frequency to 96Mhz , core voltage to 1.1v */
    BOARD_BootClockHSRUN();
#else
    /* Set Core frequency to 48Mhz , core voltage to 1.0v */
    BOARD_BootClockRUN();
#endif

#if defined(gBoardUseFro32k_d) && (gBoardUseFro32k_d > 0)
    SFC_GetDefaultConfig(&sfcConfig);
    SFC_Init(&sfcConfig);
#else
    /* Initialize and start osc32k
     * Note: osc32k is not yet selected as 32k clock source, the switch is done
     * when calling PLATFORM_SwitchToOsc32k a bit later.
     * This allows to perform other initialization task before the 32k clock is
     * actually needed */
    (void)PLATFORM_InitOsc32K();
#endif

    /* Initialize DCDC and apply optimized configuration */
    BOARD_InitDcdc();

#if defined(gDebugConsoleEnable_d) && (gDebugConsoleEnable_d == 1)
    BOARD_InitDebugConsole();
#endif

    /*OM15082 D1 D2 D3*/
    BOARD_InitPinLED1_OM15082();
    BOARD_InitPinLED2_OM15082();
    BOARD_InitPinLED3_OM15082();
	
    /*K32W148EVK - PTC6 : USER*/
    BOARD_InitPinButton1();

    /*OM15082 - SW1 : Button1 SW2 : Button2 SW3 : Button3 SW4 : Button4*/ 
    BOARD_InitPinButton1_OM15082(); 
    BOARD_InitPinButton2_OM15082();  
    BOARD_InitPinButton3_OM15082();  
    BOARD_InitPinButton4_OM15082();  

    /* Pins and clocks for serial, timers, ... are initialized in board_comp.c api
     * see BOARD_InitSerialManager for example */
}
/*${function:end}*/
