/*****************************************************************************
 * Copyright  2018 NXP
 * All rights reserved
 *

 * SPDX-License-Identifier: LA_OPT_BASE_LICENSE v24 May 2018
 *****************************************************************************/
/*****************************************************************************
 *
 * MODULE:             JN-AN-1257
 *
 * COMPONENT:          app_start.c
 *
 * DESCRIPTION:        Window Covering Initialisation
 *
 *****************************************************************************
 *
 * This software is owned by NXP B.V. and/or its supplier and is protected
 * under applicable copyright laws. All rights are reserved. We grant You,
 * and any third parties, a license to use this software solely and
 * exclusively on NXP products [NXP Microcontrollers such as JN5168, JN5169,
 * JN5179, JN5189].
 * You, and any third parties must reproduce the copyright and warranty notice
 * and any other legend of ownership on each copy or partial copy of the
 * software.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright NXP B.V. 2017-2020. All rights reserved
 *
 ****************************************************************************/

/****************************************************************************/
/***        Include files                                                 ***/
/****************************************************************************/

#include <jendefs.h>
#ifndef K32W148EVK
#include "fsl_wwdt.h"
#include "fsl_iocon.h"
#include "fsl_gpio.h"
#include "fsl_power.h"
#else
#include "fsl_wdog32.h"
#include "fsl_gpio.h"
#include "fsl_cmc.h"
#include "fsl_wuu.h"
#include "fsl_spc.h"
#include "fsl_pm_core.h"
#include "fsl_pm_board.h"
#include "fsl_vbat.h"
#include "fsl_lptmr.h"
#endif

#include "pin_mux.h"
#include "app.h"

#ifdef APP_LOW_POWER_API
#include "PWR_interface.h"
#ifdef K32W148EVK
#include "pwrm.h"
#endif
#else
#include "pwrm.h"
#endif

#include "pdum_nwk.h"
#include "pdum_apl.h"
#include "pdum_gen.h"
#include "PDM.h"
#include "dbg.h"
#include "zps_gen.h"
#include "zps_apl.h"
#include "zps_apl_af.h"
#include "zps_apl_zdo.h"
#include "string.h"
#include "app_window_covering.h"
#include "zcl_options.h"
#include "app_common.h"
#include "app_main.h"
#include "portmacro.h"
#ifndef K32W148EVK
#include "DebugExceptionHandlers_jn518x.h"
#endif
#ifdef APP_NTAG_NWK
#include "app_ntag.h"
#include "app_ntag_reset.h"
#if (JENNIC_CHIP_FAMILY == JN518x)
#include "fsl_power.h"
#endif
#endif

/****************************************************************************/
/***        Macro Definitions                                             ***/
/****************************************************************************/

#ifdef DEBUG_APP
    #define TRACE_APP   TRUE
#else
    #define TRACE_APP   FALSE
#endif

#define HALT_ON_EXCEPTION   FALSE

/****************************************************************************/
/***        Type Definitions                                              ***/
/****************************************************************************/

/****************************************************************************/
/***        Local Function Prototypes                                     ***/
/****************************************************************************/

PRIVATE void APP_vInitialise(void);
PRIVATE void vfExtendedStatusCallBack(ZPS_teExtendedStatus eExtendedStatus);

/****************************************************************************/
/***        Exported Variables                                            ***/
/****************************************************************************/

extern void *_stack_low_water_mark;

/****************************************************************************/
/***        Local Variables                                               ***/
/****************************************************************************/

/****************************************************************************/
/***        Exported Functions                                            ***/
/****************************************************************************/
extern void OSA_TimeInit(void);
/****************************************************************************
 *
 * NAME: vAppMain
 *
 * DESCRIPTION:
 * Entry point for application from a cold start.
 *
 * RETURNS:
 * Never returns.
 *
 ****************************************************************************/
PUBLIC void vAppMain(void)
{
#ifndef K32W148EVK
    wwdt_config_t  config;

    /* Initialise exception handlers for debugging */
    vDebugExceptionHandlersInitialise();

    /* Catch resets due to watchdog timer expiry */
    WWDT_GetDefaultConfig(&config);

    if (((PMC->RESETCAUSE) & PMC_RESETCAUSE_WDTRESET_MASK) == PMC_RESETCAUSE_WDTRESET_MASK)
    {
        /* Enable the WWDT clock */
        CLOCK_EnableClock(kCLOCK_WdtOsc);
        RESET_PeripheralReset(kWWDT_RST_SHIFT_RSTn);
        WWDT_Deinit(WWDT);
        DBG_vPrintf(TRACE_APP, "APP: Watchdog timer has reset device!\r\n");
        POWER_ClearResetCause();
        while(1) {
            volatile uint32 u32Delay;

            for (u32Delay = 0; u32Delay < 100000; u32Delay++);
            GPIO_PinWrite(GPIO, APP_BOARD_GPIO_PORT, APP_BASE_BOARD_LED1_PIN, 0);
            GPIO_PinWrite(GPIO, APP_BOARD_GPIO_PORT, APP_BASE_BOARD_LED2_PIN, 0);
            for (u32Delay = 0; u32Delay < 100000; u32Delay++);
            GPIO_PinWrite(GPIO, APP_BOARD_GPIO_PORT, APP_BASE_BOARD_LED1_PIN, 1);
            GPIO_PinWrite(GPIO, APP_BOARD_GPIO_PORT, APP_BASE_BOARD_LED2_PIN, 1);
        }
    }
#else /*K32W148EVK*/
#ifdef WATCHDOG_ALLOWED
    uint32_t temp;
    /* When system is boot up, WDOG32 is disabled. We must wait for at least 2.5
     * periods of wdog32 clock to reconfigure wodg32. So Delay a while to wait for
     * the previous configuration taking effect. */
    for (temp = 0; temp < 100000U; temp++)
    {
        __NOP();
    }
    if(CMC_GetSystemResetStatus(CMC0) & kCMC_Watchdog0Reset)
    {
         DBG_vPrintf(TRACE_APP, "APP: Watchdog timer has reset device!\n");
#if HALT_ON_EXCEPTION
         WDOG32_Deinit(wdog32_base);
         while (1);
#else
        /* Clear flag */
        CMC_ClearSystemResetInterruptFlags(CMC0,kCMC_Watchdog0Reset);
#endif    
    }
    wdog32_config_t config;
    WDOG32_GetDefaultConfig(&config);
    config.prescaler = kWDOG32_ClockPrescalerDivide256;
    config.timeoutValue = 125*10; /*10s*/
    WDOG32_Init(wdog32_base, &config);
    WDOG32_Refresh(wdog32_base);
#endif
#endif

    /* idle task commences here */
    DBG_vPrintf(TRUE,"\r\n");
    DBG_vPrintf(TRUE, "***********************************************\r\n");
    DBG_vPrintf(TRUE, "* WINDOW COVERING RESET                       *\r\n");
    DBG_vPrintf(TRUE, "***********************************************\r\n");
#ifndef K32W148EVK
    DBG_vPrintf(TRUE, "SYSCON->MEMORYREMAP = x%08x\r\n", SYSCON->MEMORYREMAP);
#endif

    DBG_vPrintf(TRACE_APP, "APP: Entering APP_vInitResources()\n");
    APP_vInitResources();

    DBG_vPrintf(TRACE_APP, "APP: Entering APP_vInitialise()\n");
    APP_vInitialise();

#if (defined APP_NTAG_NWK)
    DBG_vPrintf(TRACE_APP, "\nAPP: Entering APP_vNtagPdmLoad()");
    /* Didn't start BDB using PDM data ? */
    if (FALSE == APP_bNtagPdmLoad())
#endif
    {
        DBG_vPrintf(TRACE_APP, "APP: Entering BDB_vStart()\n");
        BDB_vStart();
    }

#ifdef APP_NTAG_NWK
#if (JENNIC_CHIP_FAMILY == JN518x)
    /* Not waking from deep sleep ? */
    if (0 == (POWER_GetResetCause() & (RESET_WAKE_DEEP_PD | RESET_WAKE_PD)))
#else
    /* Not waking from deep sleep ? */
    if (0 == (u16AHI_PowerStatus() & (1 << 11)))
#endif
    {
        DBG_vPrintf(TRACE_APP, "\nAPP: Entering APP_vNtagStart()");
	 vResetNTAGAddress();  
        APP_vNtagStart(WINDOWCOVERING_APPLICATION_ENDPOINT);
    }
#endif

#ifndef K32W148EVK
    /*switch to XTAL 32M for better timing accuracy*/
    SYSCON -> MAINCLKSEL    =  2;  /* 32 M XTAL */
    SystemCoreClockUpdate();
#endif
    OSA_TimeInit();
}

/****************************************************************************
 *
 * NAME: vAppRegisterPWRCallbacks
 *
 * DESCRIPTION:
 * Power manager callback.
 * Called to allow the application to register
 * sleep and wake callbacks.
 *
 * RETURNS:
 * void
 *
 ****************************************************************************/
void vAppRegisterPWRCallbacks(void)
{
    /* nothing to register as device does not sleep */
}

/****************************************************************************/
/***        Local Functions                                               ***/
/****************************************************************************/

/****************************************************************************
 *
 * NAME: APP_vInitialise
 *
 * DESCRIPTION:
 * Initialises Zigbee stack, hardware and application.
 *
 * RETURNS:
 * void
 *
 ****************************************************************************/
PRIVATE void APP_vInitialise(void)
{
    /* Initialise Power Manager even on non-sleeping nodes as it allows the
     * device to doze when in the idle task */
#ifdef APP_LOW_POWER_API
#ifndef K32W148EVK
    (void) PWR_ChangeDeepSleepMode(PWR_E_SLEEP_OSCON_RAMON);
    PWR_Init();
    PWR_vForceRadioRetention(TRUE);
#endif
#else
    PWRM_vInit(E_AHI_SLEEP_OSCON_RAMON);
    PWRM_vForceRadioRetention(TRUE);
#endif

    /* Initialise the Persistent Data Manager */
    PDM_eInitialise(1200, 63, NULL);

    /* Initialise Protocol Data Unit Manager */
    PDUM_vInit();

    ZPS_vExtendedStatusSetCallback(vfExtendedStatusCallBack);

#ifndef K32W148EVK
    /* Update radio temperature (loading calibration) */
    APP_vRadioTempUpdate(TRUE);
#endif

    /* Initialise application */
    APP_vInitialiseRouter();
}

/****************************************************************************
 *
 * NAME: vfExtendedStatusCallBack
 *
 * DESCRIPTION:
 * Callback from stack on extended error situations.
 *
 * RETURNS:
 * void
 *
 ****************************************************************************/
PRIVATE void vfExtendedStatusCallBack (ZPS_teExtendedStatus eExtendedStatus)
{
    DBG_vPrintf(TRUE,"ERROR: Extended status 0x%02x\r\n", eExtendedStatus);
}

#ifndef K32W148EVK
/****************************************************************************
 *
 * NAME: hardware_init
 *
 * DESCRIPTION:
 *
 *
 * RETURNS:
 * void
 *
 ****************************************************************************/
void hardware_init(void)
{
    APP_vSetUpHardware();
}
#endif
/****************************************************************************/
/***        END OF FILE                                                   ***/
/****************************************************************************/
