/*****************************************************************************
 * Copyright  2018 NXP
 * All rights reserved
 *

 * SPDX-License-Identifier: LA_OPT_BASE_LICENSE v24 May 2018
 *****************************************************************************/
/*****************************************************************************
 *
 * MODULE:          JN-AN-1243 Base Device Application
 *
 * COMPONENT:       app_ntag_reset.h
 *
 * DESCRIPTION:     Application layer for NTAG (Installation Code encryption)
 *
 ****************************************************************************
 *
 * This software is owned by NXP B.V. and/or its supplier and is protected
 * under applicable copyright laws. All rights are reserved. We grant You,
 * and any third parties, a license to use this software solely and
 * exclusively on NXP products [NXP Microcontrollers such as JN5168, JN5169,
 * JN5179, JN5189].
 * You, and any third parties must reproduce the copyright and warranty notice
 * and any other legend of ownership on each copy or partial copy of the
 * software.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright NXP B.V. 2017-2018. All rights reserved
 *
 ***************************************************************************/
/*!
 * \file  app_ntag_reset.h
 *
 * \brief ZigBee 3.0 NTAG network commissioning
 *
 * app_ntag_reset.h contains application APIs that can be used to commission ZigBee 3.0
 * devices into a network using NFC to write network information into an NTAG. The
 * code in app_ntag.h makes use of libNTAG to interact with the NTAG.
 *
 * The changes required to add these features to ZigBee 3.0 applications are
 * described in the sections on the individual functions and have already been made
 * to the ZigBee 3.0 Application Notes.
 *
 * The changes in the source code for network commissioning are
 * all wrapped by \#ifdef APP_NTAG_NWK and the NTAG functionality can be disabled
 * in the makefiles or on the command line by setting APP_NTAG_NWK = 0.
 *
 * The changes in the source code for OTA over NFC are
 * all wrapped by \#ifdef APP_NTAG_OTA and the NTAG functionality can be disabled
 * in the makefiles or on the command line by setting APP_NTAG_OTA = 0.
 *
 ***************************************************************************/


/****************************************************************************/
/***        Include Files                                                 ***/
/****************************************************************************/
#include <jendefs.h>
/****************************************************************************/
/***        Macro Definitions                                             ***/
/****************************************************************************/

/****************************************************************************/
/***        Type Definitions                                              ***/
/****************************************************************************/
/****************************************************************************
 *
 * NAME:       iNTAG_ReadBlock
 *
 * DESCRIPTION:
 * Function to read a block of NTAG EEPROM over I2C
 *
 * RETURNS:
 * int - returns 0 if OK
 *
 ****************************************************************************/
PUBLIC int iNTAG_ReadBlock(uint32_t slvAddr, uint32_t blockid, uint8_t *data);

/****************************************************************************
 *
 * NAME:       iNTAG_WriteBlock
 *
 * DESCRIPTION:
 * Function to write a block of NTAG EEPROM over I2C
 *
 * RETURNS:
 * int - returns 0 if OK
 *
 ****************************************************************************/
PUBLIC int iNTAG_WriteBlock(uint32_t slvAddr, uint32_t blockid, const uint8_t *data);

/****************************************************************************
 *
 * NAME:       iNTAG_DumpBlock
 *
 * DESCRIPTION:
 * Function to dump a block of NTAG EEPROM to the UART
 *
 * RETURNS:
 * int - returns 0 if OK
 *
 ****************************************************************************/
PUBLIC int iNTAG_DumpBlock(uint32_t u32BlockId, uint32_t slvAddr);

/****************************************************************************
 *
 * NAME:       vResetNTAGAddress
 *
 * DESCRIPTION:
 * Re-programs NTAG slave address to 0x55 if accidentally reprogrammed
 *
 * Value always read as 0x4 so Reading and writing the same values back will
 * change the address
 *
 * RETURNS:
 * void
 *
 ****************************************************************************/
PUBLIC void vResetNTAGAddress(void );

/****************************************************************************/
/***        Exported Functions                                            ***/
/****************************************************************************/
/****************************************************************************/
/***        Exported Variables                                            ***/
/****************************************************************************/

/****************************************************************************/
/***        END OF FILE                                                   ***/
/****************************************************************************/
