/*****************************************************************************
 * Copyright  2018 NXP
 * All rights reserved
 *

 * SPDX-License-Identifier: LA_OPT_BASE_LICENSE v24 May 2018
 *****************************************************************************/
/****************************************************************************
 *
 * MODULE              JN-AN-1244 ZigBee Light Link Application
 *
 * COMPONENT:          app_scenes.c
 *
 * DESCRIPTION         Application Scenes save and load functionality
 *
 ****************************************************************************
 *
 * This software is owned by NXP B.V. and/or its supplier and is protected
 * under applicable copyright laws. All rights are reserved. We grant You,
 * and any third parties, a license to use this software solely and
 * exclusively on NXP products [NXP Microcontrollers such as JN5168, JN5169,
 * JN5179, JN5189].
 * You, and any third parties must reproduce the copyright and warranty notice
 * and any other legend of ownership on each copy or partial copy of the
 * software.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright NXP B.V. 2016-2018. All rights reserved
 *
 ***************************************************************************/

/****************************************************************************/
/***        Include files                                                 ***/
/****************************************************************************/

#include <jendefs.h>
#include <string.h>
#include "pdum_gen.h"
#include "PDM.h"
#include "dbg.h"
#include "zps_gen.h"
#include "PDM_IDs.h"
#include "zcl_options.h"
#include "zps_apl_af.h"

#include "app_common.h"

#include "Scenes.h"
#include "app_scenes.h"
#include "app_zcl_task.h"
#ifdef CLD_GROUPS
#include "Groups_internal.h"
#endif
/****************************************************************************/
/***        Macro Definitions                                             ***/
/****************************************************************************/
#ifdef DEBUG_APP_SCENES
    #define TRACE_APP_SCENES   TRUE
#else
    #define TRACE_APP_SCENES   FALSE
#endif

#define SCENES_SEARCH_GROUP_ID      (1 << 0)
#define SCENES_SEARCH_SCENE_ID      (1 << 1)


/****************************************************************************/
/***        Type Definitions                                              ***/
/****************************************************************************/
typedef struct
{
    uint8   u8SearchOptions;
    uint16  u16GroupId;
    uint8   u8SceneId;
} tsSearchParameter;

/****************************************************************************/
/***        Local Function Prototypes                                     ***/
/****************************************************************************/


/****************************************************************************/
/***        Exported Variables                                            ***/
/****************************************************************************/

/****************************************************************************/
/***        Local Variables                                               ***/
/****************************************************************************/
#if (defined CLD_SCENES) && (defined SCENES_SERVER)
PRIVATE tsAPP_ScenesCustomData sScenesCustomData;
#endif


/****************************************************************************/
/***        Exported Functions                                            ***/
/****************************************************************************/

#if (defined CLD_SCENES) && (defined SCENES_SERVER)
/****************************************************************************
*
* NAME: vSaveScenesNVM
*
* DESCRIPTION:
* To save scenes data to EEPROM
*
* RETURNS:
* void
*
****************************************************************************/
PUBLIC void vSaveScenesNVM(void)
{
    uint8 i=0, j=0;
    tsCLD_ScenesTableEntry *psTableEntry;
    PDM_teStatus eStatus;

    DBG_vPrintf(TRACE_APP_SCENES, "\r\nAPP_SCENES: vSaveScenesNVM()");

    /* Before saving data again clear off the earlier entries and recopy all the scenes */
    memset(&sScenesCustomData,0,sizeof(tsAPP_ScenesCustomData));
    psTableEntry = (tsCLD_ScenesTableEntry*)psDLISTgetHead(&sWindowCovering.sScenesServerCustomDataStructure.lScenesAllocList);
    for(i = 0; i < iDLISTnumberOfNodes(&sWindowCovering.sScenesServerCustomDataStructure.lScenesAllocList); i++)
    {
        if(psTableEntry != NULL)
        {
            DBG_vPrintf(TRACE_APP_SCENES, "\r\nAPP_SCENES: From %d to %d", i, j);
            sScenesCustomData.asScenesCustomTableEntry[j].bIsSceneValid = TRUE;
            sScenesCustomData.asScenesCustomTableEntry[j].u16GroupId = psTableEntry->u16GroupId;
            sScenesCustomData.asScenesCustomTableEntry[j].u8SceneId = psTableEntry->u8SceneId;
            sScenesCustomData.asScenesCustomTableEntry[j].u16TransitionTime = psTableEntry->u16TransitionTime;
            sScenesCustomData.asScenesCustomTableEntry[j].u16SceneDataLength = psTableEntry->u16SceneDataLength;
            memcpy(sScenesCustomData.asScenesCustomTableEntry[j].au8SceneData,psTableEntry->au8SceneData,psTableEntry->u16SceneDataLength);
            #ifdef CLD_SCENES_TABLE_SUPPORT_TRANSITION_TIME_IN_MS
                sScenesCustomData.asScenesCustomTableEntry[j].u8TransitionTime100ms = psTableEntry->u8TransitionTime100ms;
            #endif
            j++;
        }
        psTableEntry = (tsCLD_ScenesTableEntry*)psDLISTgetNext((DNODE*)psTableEntry);
    }

    eStatus = PDM_eSaveRecordData(PDM_ID_APP_SCENES_DATA, &sScenesCustomData, sizeof(tsAPP_ScenesCustomData));
    DBG_vPrintf(TRACE_APP_SCENES, "\r\nAPP_SCENES: PDM_eSaveRecordData(APP_SCENES_DATA) = %d", eStatus);
}

/****************************************************************************
*
* NAME: vLoadScenesNVM
*
* DESCRIPTION:
* To load scenes data from EEPROM
*
* RETURNS:
* void
*
****************************************************************************/
PUBLIC void vLoadScenesNVM(void)
{
    uint8 i=0;
    uint16 u16ByteRead;
    PDM_teStatus eStatus;

    DBG_vPrintf(TRACE_APP_SCENES, "\r\nAPP_SCENES: vLoadScenesNVM()");

    eStatus = PDM_eReadDataFromRecord(PDM_ID_APP_SCENES_DATA,
                                      &sScenesCustomData,
                                      sizeof(tsAPP_ScenesCustomData), &u16ByteRead);
    DBG_vPrintf(TRACE_APP_SCENES, "\r\nAPP_SCENES: PDM_eReadDataFromRecord(APP_SCENES_DATA) = %d", eStatus);

    /* initialise lists */
    vDLISTinitialise(&sWindowCovering.sScenesServerCustomDataStructure.lScenesAllocList);
    vDLISTinitialise(&sWindowCovering.sScenesServerCustomDataStructure.lScenesDeAllocList);

    for(i=0; i<CLD_SCENES_MAX_NUMBER_OF_SCENES; i++)
    {
        DBG_vPrintf(TRACE_APP_SCENES, "\r\nAPP_SCENES: To %d", i);
        /* Rebuild the scene list to avoid scene loss after re-flashing */
        if(sScenesCustomData.asScenesCustomTableEntry[i].bIsSceneValid == TRUE)
        {
             vDLISTaddToTail(&sWindowCovering.sScenesServerCustomDataStructure.lScenesAllocList,
                            (DNODE *)&sWindowCovering.sScenesServerCustomDataStructure.asScenesTableEntry[i]);
        }
        else
        {
             vDLISTaddToTail(&sWindowCovering.sScenesServerCustomDataStructure.lScenesDeAllocList,
                            (DNODE *)&sWindowCovering.sScenesServerCustomDataStructure.asScenesTableEntry[i]);
        }

        sWindowCovering.sScenesServerCustomDataStructure.asScenesTableEntry[i].u16GroupId = sScenesCustomData.asScenesCustomTableEntry[i].u16GroupId;
        sWindowCovering.sScenesServerCustomDataStructure.asScenesTableEntry[i].u8SceneId = sScenesCustomData.asScenesCustomTableEntry[i].u8SceneId;
        sWindowCovering.sScenesServerCustomDataStructure.asScenesTableEntry[i].u16TransitionTime = sScenesCustomData.asScenesCustomTableEntry[i].u16TransitionTime;
        sWindowCovering.sScenesServerCustomDataStructure.asScenesTableEntry[i].u16SceneDataLength = sScenesCustomData.asScenesCustomTableEntry[i].u16SceneDataLength;
        memcpy(sWindowCovering.sScenesServerCustomDataStructure.asScenesTableEntry[i].au8SceneData, sScenesCustomData.asScenesCustomTableEntry[i].au8SceneData,sScenesCustomData.asScenesCustomTableEntry[i].u16SceneDataLength);
#ifdef CLD_SCENES_TABLE_SUPPORT_TRANSITION_TIME_IN_MS
        sWindowCovering.sScenesServerCustomDataStructure.asScenesTableEntry[i].u8TransitionTime100ms = sScenesCustomData.asScenesCustomTableEntry[i].u8TransitionTime100ms;
#endif
    }
}
#endif


#ifdef CLD_GROUPS
/****************************************************************************
 *
 * NAME: APP_vRemoveAllGroupsAndScenes
 *
 * DESCRIPTION:
 * to remove all scenes and groups after a leave or factory reset
 *
 * RETURNS:
 * void
 *
 ****************************************************************************/
PUBLIC void APP_vRemoveAllGroupsAndScenes(void)
{
    DBG_vPrintf(TRACE_APP_SCENES, "\r\nAPP_SCENES: APP_vRemoveAllGroupsAndScenes()");
    eCLD_GroupsRemoveAllGroups(&sWindowCovering.sEndPoint,
                               &sWindowCovering.sClusterInstance.sGroupsServer,
                               (uint64)0xffffffffffffffffLL);

}
#endif

/****************************************************************************/
/***        END OF FILE                                                   ***/
/****************************************************************************/
