/*****************************************************************************
 * Copyright  2018 NXP
 * All rights reserved
 *

 * SPDX-License-Identifier: LA_OPT_BASE_LICENSE v24 May 2018
 *****************************************************************************/
/*****************************************************************************
 *
 * MODULE:             Motor Driver A4988
 *
 * COMPONENT:          MotorDriver_A4988.h
 *
 * AUTHOR:             Martin Looker
 *
 * DESCRIPTION:        Header for A4988 Stepper Motor Driver
 *
 *****************************************************************************
 *
 * This software is owned by NXP B.V. and/or its supplier and is protected
 * under applicable copyright laws. All rights are reserved. We grant You,
 * and any third parties, a license to use this software solely and
 * exclusively on NXP products  [NXP Microcontrollers such as JN5189, JN%179,
 * JN5168, JN5164, JN5161].
 * You, and any third parties must reproduce the copyright and warranty notice
 * and any other legend of ownership on each  copy or partial copy of the software.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright NXP B.V. 2019. All rights reserved
 *
 ****************************************************************************/
#ifndef DRIVERMOTOR_H_
#define DRIVERMOTOR_H_

#if defined __cplusplus
extern "C" {
#endif

#include <jendefs.h>
#include "app.h"

/****************************************************************************/
/***        Macro Definitions                                             ***/
/****************************************************************************/
/* Define one of these */
#define DM_ULN2003

/* ULN2003 settings */
#ifdef DM_ULN2003
#define DM_OUTPUT_PIN_A     18
#define DM_OUTPUT_PIN_B      2
#define DM_OUTPUT_PIN_C     11
#define DM_OUTPUT_PIN_D     10
#define DM_STEPS_PER_CM    321
#define DM_STEP_HARD_MIN     0                    /* Hard limit for minimum step */
#define DM_STEP_HARD_MAX  (DM_STEPS_PER_CM * 100) /* Hard limit for maximum step */
#endif

/* Bit definitions for Mode (match bits in window covering mode attribute that affect motor) */
#define DM_MODE_BIT_REVERSED    (1 << 0)         /* Motor direction is reversed */
#define DM_MODE_BIT_CALIBRATION (1 << 1)         /* In calibration mode (only repsect hard limits) */
#define DM_MODE_BIT_MAINTENANCE (1 << 2)         /* Maintenence do not operate motor */

/****************************************************************************/
/***        Type Definitions                                              ***/
/****************************************************************************/
/* Motor driver modes */
typedef enum
{
    E_DM_DIR_STOP,              /* Stopped */
    E_DM_DIR_FORWARD,           /* Forward (increasing steps) */
    E_DM_DIR_BACKWARD           /* Backward (decreasing steps) */
} teDM_Dir;

/* Motor driver state */
typedef struct
{
    teDM_Dir   eDir;            /* Current direction */
    uint8     u8Mode;           /* Current mode */
    uint32    u32Step;          /* Current step position */
    uint32    u32StepSoftMin;   /* Soft limit for minimum step value */
    uint32    u32StepSoftMax;   /* Soft limit for maximum step value */
    uint32    u32Activations;   /* Number of activations */
} tsDM_State;

/****************************************************************************/
/***        Exported Functions                                            ***/
/****************************************************************************/
PUBLIC void     vDM_Initialise(tsDM_State *psDmStateInit); /* Initialisation */
PUBLIC teDM_Dir eDM_Stop(void);     /* Stop */
PUBLIC teDM_Dir eDM_Forward(void);  /* Forward (increasing steps) */
PUBLIC teDM_Dir eDM_Backward(void); /* Backward (decreasing steps) */
PUBLIC teDM_Dir eDM_GoToTarget(uint32 u32StepTarget, bool_t *pbValid); /* Move to target step */
PUBLIC teDM_Dir eDM_Tick(void);     /* Timer tick function */

#if defined __cplusplus
}
#endif

/****************************************************************************/
/***        END OF FILE                                                   ***/
/****************************************************************************/

#endif /* DRIVERMOTOR_H_ */
